@@ -201,20 +201,16 @@ static void
 gst_alsasrc_class_init (GstAlsaSrcClass * klass)
 {
   GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseSrcClass *gstbasesrc_class;
-  GstBaseAudioSrcClass *gstbaseaudiosrc_class;
   GstAudioSrcClass *gstaudiosrc_class;
 
   gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
   gstbasesrc_class = (GstBaseSrcClass *) klass;
-  gstbaseaudiosrc_class = (GstBaseAudioSrcClass *) klass;
   gstaudiosrc_class = (GstAudioSrcClass *) klass;
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_alsasrc_finalize);
-  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_alsasrc_get_property);
-  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_alsasrc_set_property);
+  gobject_class->finalize = gst_alsasrc_finalize;
+  gobject_class->get_property = gst_alsasrc_get_property;
+  gobject_class->set_property = gst_alsasrc_set_property;
 
   gstbasesrc_class->get_caps = GST_DEBUG_FUNCPTR (gst_alsasrc_getcaps);
 
