@@ -2,7 +2,8 @@
 #define __GST_TEXT_OVERLAY_H__
 
 #include <gst/gst.h>
-#include <pango/pangoft2.h>
+#include <gst/video/video.h>
+#include <pango/pangocairo.h>
 
 G_BEGIN_DECLS
 
@@ -91,7 +92,7 @@ struct _GstTextOverlay {
     GstPad                  *text_sinkpad;
     GstPad                  *srcpad;
 
-    GstSegment              *segment;
+    GstSegment               segment;
     GstSegment               text_segment;
     GstBuffer               *text_buffer;
     gboolean                text_linked;
@@ -109,7 +110,7 @@ struct _GstTextOverlay {
     gint                     height;
     gint                     fps_n;
     gint                     fps_d;
-    guint32                  format;
+    GstVideoFormat           format;
 
     GstTextOverlayVAlign     valign;
     GstTextOverlayHAlign     halign;
@@ -126,15 +127,20 @@ struct _GstTextOverlay {
     gboolean                 wait_text;
 
     PangoLayout             *layout;
-    FT_Bitmap                bitmap;
-    gint                     bitmap_buffer_size;
+    gdouble                  shadow_offset;
+    gdouble                  outline_offset;
+    guchar                  *text_image;
+    gint                     image_width;
+    gint                     image_height;
     gint                     baseline_y;
 
+    gboolean                 auto_adjust_size;
     gboolean                 need_render;
 
     gint                     shading_value;  /* for timeoverlay subclass */
 
     gboolean                 have_pango_markup;
+    gboolean                 use_vertical_render;
 };
 
 struct _GstTextOverlayClass {
